@@ -51,6 +51,7 @@
 #include "p11-kit.h"
 #include "private.h"
 #include "proxy.h"
+#include "rpc.h"
 #include "virtual.h"
 
 #include <sys/stat.h>
@@ -172,6 +173,13 @@ static struct _Shared {
 	p11_dict *config;
 } gl = { NULL, NULL };
 
+/* These are global variables to be overridden in tests */
+const char *p11_config_system_file = P11_SYSTEM_CONFIG_FILE;
+const char *p11_config_user_file = P11_USER_CONFIG_FILE;
+const char *p11_config_package_modules = P11_PACKAGE_CONFIG_MODULES;
+const char *p11_config_system_modules = P11_SYSTEM_CONFIG_MODULES;
+const char *p11_config_user_modules = P11_USER_CONFIG_MODULES;
+
 /* -----------------------------------------------------------------------------
  * P11-KIT FUNCTIONALITY
  */
@@ -370,7 +378,7 @@ load_module_from_file_inlock (const char *name,
 	/* If same module was loaded previously, just take over config */
 	if (prev != NULL) {
 		if (!name || prev->name || prev->config)
-			p11_debug ("duplicate module %s, using previous", name);
+			p11_debug ("duplicate module %s, using previous", path);
 		free_module_unlocked (mod);
 		mod = prev;
 
@@ -384,6 +392,36 @@ load_module_from_file_inlock (const char *name,
 	return CKR_OK;
 }
 
+static CK_RV
+setup_module_for_remote_inlock (const char *name,
+                                const char *remote,
+                                Module **result)
+{
+	p11_rpc_transport *rpc;
+	Module *mod;
+
+	p11_debug ("remoting module %s using: %s", name, remote);
+
+	mod = alloc_module_unlocked ();
+	return_val_if_fail (mod != NULL, CKR_HOST_MEMORY);
+
+	rpc = p11_rpc_transport_new (&mod->virt, remote, name);
+	if (rpc == NULL) {
+		free_module_unlocked (mod);
+		return CKR_DEVICE_ERROR;
+	}
+
+	mod->loaded_module = rpc;
+	mod->loaded_destroy = p11_rpc_transport_free;
+
+	/* This takes ownership of the module */
+	if (!p11_dict_set (gl.modules, mod, mod))
+		return_val_if_reached (CKR_HOST_MEMORY);
+
+	*result = mod;
+	return CKR_OK;
+}
+
 static int
 is_list_delimiter (char ch)
 {
@@ -444,9 +482,12 @@ take_config_and_load_module_inlock (char **name,
                                     p11_dict **config,
                                     bool critical)
 {
-	const char *filename;
+	const char *filename = NULL;
+	const char *remote = NULL;
+	char *value = NULL;
+	CK_RV rv = CKR_OK;
+	bool isolated;
 	Module *mod;
-	CK_RV rv;
 
 	assert (name);
 	assert (*name);
@@ -454,17 +495,48 @@ take_config_and_load_module_inlock (char **name,
 	assert (*config);
 
 	if (!is_module_enabled_unlocked (*name, *config))
-		return CKR_OK;
+		goto out;
 
-	filename = p11_dict_get (*config, "module");
-	if (filename == NULL) {
-		p11_debug ("no module path for module, skipping: %s", *name);
-		return CKR_OK;
+	remote = p11_dict_get (*config, "remote");
+	if (remote == NULL) {
+		filename = p11_dict_get (*config, "module");
+		if (filename == NULL) {
+			p11_debug ("no module path for module, skipping: %s", *name);
+			goto out;
+		}
 	}
 
-	rv = load_module_from_file_inlock (*name, filename, &mod);
-	if (rv != CKR_OK)
-		return CKR_OK;
+	/* The 'isolated' setting is just a simple way to configure remote */
+	isolated = _p11_conf_parse_boolean (p11_dict_get (*config, "isolated"), false);
+	if (isolated) {
+		if (remote) {
+			p11_message ("ignoring 'isolated' on module '%s' because 'remote' is set", *name);
+			isolated = false;
+		} else {
+			if (asprintf (&value, "|" BINDIR "/p11-kit remote '%s'", filename) < 0)
+				return_val_if_reached (CKR_DEVICE_ERROR);
+			remote = value;
+		}
+	}
+
+	if (remote != NULL) {
+		rv = setup_module_for_remote_inlock (*name, remote, &mod);
+		if (rv != CKR_OK)
+			goto out;
+
+	} else {
+
+		rv = load_module_from_file_inlock (*name, filename, &mod);
+		if (rv != CKR_OK)
+			goto out;
+
+		/*
+		 * We support setting of CK_C_INITIALIZE_ARGS.pReserved from
+		 * 'x-init-reserved' setting in the config. This only works with specific
+		 * PKCS#11 modules, and is non-standard use of that field.
+		 */
+		mod->init_args.pReserved = p11_dict_get (*config, "x-init-reserved");
+	}
 
 	/* Take ownership of thes evariables */
 	p11_dict_free (mod->config);
@@ -475,14 +547,9 @@ take_config_and_load_module_inlock (char **name,
 	*name = NULL;
 	mod->critical = critical;
 
-	/*
-	 * We support setting of CK_C_INITIALIZE_ARGS.pReserved from
-	 * 'x-init-reserved' setting in the config. This only works with specific
-	 * PKCS#11 modules, and is non-standard use of that field.
-	 */
-	mod->init_args.pReserved = p11_dict_get (mod->config, "x-init-reserved");
-
-	return CKR_OK;
+out:
+	free (value);
+	return rv;
 }
 
 static CK_RV
@@ -501,16 +568,16 @@ load_registered_modules_unlocked (void)
 		return CKR_OK;
 
 	/* Load the global configuration files */
-	config = _p11_conf_load_globals (P11_SYSTEM_CONFIG_FILE, P11_USER_CONFIG_FILE, &mode);
+	config = _p11_conf_load_globals (p11_config_system_file, p11_config_user_file, &mode);
 	if (config == NULL)
 		return CKR_GENERAL_ERROR;
 
 	assert (mode != CONF_USER_INVALID);
 
 	configs = _p11_conf_load_modules (mode,
-	                                  P11_PACKAGE_CONFIG_MODULES,
-	                                  P11_SYSTEM_CONFIG_MODULES,
-	                                  P11_USER_CONFIG_MODULES);
+	                                  p11_config_package_modules,
+	                                  p11_config_system_modules,
+	                                  p11_config_user_modules);
 	if (configs == NULL) {
 		rv = CKR_GENERAL_ERROR;
 		p11_dict_free (config);
